"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const apputils_1 = require("@jupyterlab/apputils");
const mainmenu_1 = require("@jupyterlab/mainmenu");
const notebook_1 = require("@jupyterlab/notebook");
const cells_1 = require("@jupyterlab/cells");
const coreutils_1 = require("@jupyterlab/coreutils");
const services_1 = require("@jupyterlab/services");
const fileeditor_1 = require("@jupyterlab/fileeditor");
require("../style/index.css");
const PLUGIN_NAME = "jupyterlab_code_formatter";
function request(path, method, body, settings) {
    const fullUrl = coreutils_1.URLExt.join(settings.baseUrl, PLUGIN_NAME, path);
    return services_1.ServerConnection.makeRequest(fullUrl, { body, method }, settings).then((response) => {
        if (response.status !== 200) {
            return response.text().then((data) => {
                throw new services_1.ServerConnection.ResponseError(response, data);
            });
        }
        return response.text();
    });
}
class JupyterLabCodeFormatter {
    constructor(app, tracker, palette, settingRegistry, menu, editorTracker) {
        this.working = false;
        this.pythonCommands = ["black", "yapf", "autopep8", "isort"].map(name => `${PLUGIN_NAME}:${name}`);
        this.rCommands = ["formatR", "styler"].map(name => `${PLUGIN_NAME}:${name}`);
        this.app = app;
        this.tracker = tracker;
        this.editorTracker = editorTracker;
        this.palette = palette;
        this.settingRegistry = settingRegistry;
        this.menu = menu;
        this.setupSettings();
        // tslint:disable-next-line:no-console
        request("formatters", "GET", null, services_1.ServerConnection.defaultSettings).then((data) => {
            const formatters = JSON.parse(data).formatters;
            let menuGroup = [];
            Object.keys(formatters).forEach((formatter) => {
                if (formatters[formatter].enabled) {
                    const command = `${PLUGIN_NAME}:${formatter}`;
                    this.setupButton(formatter, formatters[formatter].label, command);
                    menuGroup.push({ command });
                }
            });
            this.menu.editMenu.addGroup(menuGroup);
        });
    }
    setupSettings() {
        const self = this;
        Promise.all([this.settingRegistry.load(`@ryantam626/${PLUGIN_NAME}:settings`)]).then(([settings]) => {
            function onSettingsUpdated(jsettings) {
                self.config = jsettings.composite;
            }
            settings.changed.connect(onSettingsUpdated);
            onSettingsUpdated(settings);
        }).catch((reason) => console.error(reason.message));
    }
    maybeFormatCodecell(formatterName) {
        // TODO: Check current kernel is of appropriate kernel
        const editorWidget = this.editorTracker.currentWidget;
        if (this.working) {
            return;
        }
        if (editorWidget && editorWidget.content !== null && editorWidget.content.isVisible) {
            this.working = true;
            const editor = editorWidget.content.editor;
            const code = editor.model.value.text;
            request("format", "POST", JSON.stringify({
                code: code,
                formatter: formatterName,
                options: this.config[formatterName],
            }), services_1.ServerConnection.defaultSettings).then((data) => {
                this.editorTracker.currentWidget.content.editor.model.value.text = JSON.parse(data);
                this.working = false;
            }).catch(() => {
                this.working = false;
                // tslint:disable-next-line:no-console
                console.error("Something went wrong :(");
            });
        }
        else if (this.tracker.activeCell instanceof cells_1.CodeCell) {
            this.working = true;
            request("format", "POST", JSON.stringify({
                code: this.tracker.activeCell.model.value.text,
                formatter: formatterName,
                options: this.config[formatterName],
            }), services_1.ServerConnection.defaultSettings).then((data) => {
                this.tracker.activeCell.model.value.text = JSON.parse(data);
                this.working = false;
            }).catch(() => {
                this.working = false;
                // tslint:disable-next-line:no-console
                console.error("Something went wrong :(");
            });
        }
    }
    setupButton(name, label, command) {
        this.app.commands.addCommand(command, {
            execute: () => {
                this.maybeFormatCodecell(name);
            },
            isVisible: () => {
                let widget = this.app.shell.currentWidget;
                // TODO: handle other languages other than Python
                let editorWidget = this.editorTracker.currentWidget;
                let notebookWidget = this.tracker.currentWidget;
                return widget && ((this.pythonCommands.some(cmd => cmd === command) && editorWidget && widget === editorWidget &&
                    coreutils_1.PathExt.extname(editorWidget.context.path).toLowerCase() === '.py') ||
                    (this.rCommands.some(cmd => cmd === command) && editorWidget && widget === editorWidget &&
                        coreutils_1.PathExt.extname(editorWidget.context.path).toLowerCase() === '.r') ||
                    (notebookWidget && widget === notebookWidget));
            },
            label,
        });
        this.palette.addItem({ command, category: "JupyterLab Code Formatter" });
    }
}
/**
 * Initialization data for the jupyterlab_code_formatter extension.
 */
const extension = {
    activate: (app, palette, tracker, settingRegistry, menu, editorTracker) => {
        // tslint:disable-next-line:no-unused-expression
        new JupyterLabCodeFormatter(app, tracker, palette, settingRegistry, menu, editorTracker);
    },
    autoStart: true,
    id: PLUGIN_NAME,
    requires: [apputils_1.ICommandPalette, notebook_1.INotebookTracker, coreutils_1.ISettingRegistry, mainmenu_1.IMainMenu, fileeditor_1.IEditorTracker],
};
exports.default = extension;
